@@ -1,13 +1,13 @@
-# sybase/base.py
-# Copyright (C) 2010-2019 the SQLAlchemy authors and contributors
+# dialects/sybase/base.py
+# Copyright (C) 2010-2024 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 # get_select_precolumns(), limit_clause() implementation
 # copyright (C) 2007 Fisch Asset Management
-# AG http://www.fam.ch, with coding by Alexander Houben
+# AG https://www.fam.ch, with coding by Alexander Houben
 # alexander.houben@thor-solutions.ch
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """
 
@@ -16,9 +16,14 @@
 
 .. note::
 
-    The Sybase dialect functions on current SQLAlchemy versions
-    but is not regularly tested, and may have many issues and
-    caveats not currently handled.
+    The Sybase dialect within SQLAlchemy **is not currently supported**.
+    It is not tested within continuous integration and is likely to have
+    many issues and caveats not currently handled. Consider using the
+    `external dialect <https://github.com/gordthompson/sqlalchemy-sybase>`_
+    instead.
+
+.. deprecated:: 1.4 The internal Sybase dialect is deprecated and will be
+   removed in a future version. Use the external dialect.
 
 """
 
@@ -511,28 +516,18 @@ class SybaseSQLCompiler(compiler.SQLCompiler):
         {"doy": "dayofyear", "dow": "weekday", "milliseconds": "millisecond"},
     )
 
-    def get_select_precolumns(self, select, **kw):
-        s = select._distinct and "DISTINCT " or ""
-        # TODO: don't think Sybase supports
-        # bind params for FIRST / TOP
-        limit = select._limit
-        if limit:
-            # if select._limit == 1:
-            # s += "FIRST "
-            # else:
-            # s += "TOP %s " % (select._limit,)
-            s += "TOP %s " % (limit,)
-        offset = select._offset
-        if offset:
-            raise NotImplementedError("Sybase ASE does not support OFFSET")
-        return s
-
     def get_from_hint_text(self, table, text):
         return text
 
     def limit_clause(self, select, **kw):
-        # Limit in sybase is after the select keyword
-        return ""
+        text = ""
+        if select._limit_clause is not None:
+            text += " ROWS LIMIT " + self.process(select._limit_clause, **kw)
+        if select._offset_clause is not None:
+            if select._limit_clause is None:
+                text += " ROWS"
+            text += " OFFSET " + self.process(select._offset_clause, **kw)
+        return text
 
     def visit_extract(self, extract, **kw):
         field = self.extract_map.get(extract.field, extract.field)
@@ -569,8 +564,9 @@ class SybaseSQLCompiler(compiler.SQLCompiler):
         self, delete_stmt, from_table, extra_froms, from_hints, **kw
     ):
         """Render the DELETE .. FROM clause specific to Sybase."""
+        kw["asfrom"] = True
         return "FROM " + ", ".join(
-            t._compiler_dispatch(self, asfrom=True, fromhints=from_hints, **kw)
+            t._compiler_dispatch(self, fromhints=from_hints, **kw)
             for t in [from_table] + extra_froms
         )
 
@@ -637,6 +633,7 @@ class SybaseDialect(default.DefaultDialect):
     supports_unicode_statements = False
     supports_sane_rowcount = False
     supports_sane_multi_rowcount = False
+    supports_statement_cache = True
 
     supports_native_boolean = False
     supports_unicode_binds = False
@@ -653,6 +650,15 @@ class SybaseDialect(default.DefaultDialect):
 
     construct_arguments = []
 
+    def __init__(self, *args, **kwargs):
+        util.warn_deprecated(
+            "The Sybase dialect is deprecated and will be removed "
+            "in a future version. This dialect is superseded by the external "
+            "dialect https://github.com/gordthompson/sqlalchemy-sybase.",
+            version="1.4",
+        )
+        super(SybaseDialect, self).__init__(*args, **kwargs)
+
     def _get_default_schema_name(self, connection):
         return connection.scalar(
             text("SELECT user_name() as user_name").columns(username=Unicode)
@@ -1084,6 +1090,8 @@ class SybaseDialect(default.DefaultDialect):
         return [v["name"] for v in views]
 
     def has_table(self, connection, table_name, schema=None):
+        self._ensure_has_table_connection(connection)
+
         try:
             self.get_table_id(connection, table_name, schema)
         except exc.NoSuchTableError:
