@@ -1,4 +1,9 @@
-from .. import config
+# testing/suite/test_update_delete.py
+# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 from .. import fixtures
 from ..assertions import eq_
 from ..schema import Column
@@ -9,6 +14,7 @@ from ... import String
 
 class SimpleUpdateDeleteTest(fixtures.TablesTest):
     run_deletes = "each"
+    __requires__ = ("sane_rowcount",)
     __backend__ = True
 
     @classmethod
@@ -31,24 +37,28 @@ class SimpleUpdateDeleteTest(fixtures.TablesTest):
             ],
         )
 
-    def test_update(self):
+    def test_update(self, connection):
         t = self.tables.plain_pk
-        r = config.db.execute(t.update().where(t.c.id == 2), data="d2_new")
+        r = connection.execute(
+            t.update().where(t.c.id == 2), dict(data="d2_new")
+        )
         assert not r.is_insert
         assert not r.returns_rows
+        assert r.rowcount == 1
 
         eq_(
-            config.db.execute(t.select().order_by(t.c.id)).fetchall(),
+            connection.execute(t.select().order_by(t.c.id)).fetchall(),
             [(1, "d1"), (2, "d2_new"), (3, "d3")],
         )
 
-    def test_delete(self):
+    def test_delete(self, connection):
         t = self.tables.plain_pk
-        r = config.db.execute(t.delete().where(t.c.id == 2))
+        r = connection.execute(t.delete().where(t.c.id == 2))
         assert not r.is_insert
         assert not r.returns_rows
+        assert r.rowcount == 1
         eq_(
-            config.db.execute(t.select().order_by(t.c.id)).fetchall(),
+            connection.execute(t.select().order_by(t.c.id)).fetchall(),
             [(1, "d1"), (3, "d3")],
         )
 
